package com.jiuqi.util.inifile;

import com.jiuqi.util.StreamException;
import com.jiuqi.util.Stream;

public final class StreamIniBuffer extends IniBuffer
{
    private Stream stream;
    private int cursor;
    
    public StreamIniBuffer(final Stream aStream) {
        this.stream = aStream;
    }
    
    @Override
    public int getEntry() {
        return this.cursor;
    }
    
    @Override
    public void gotoCursor() {
        try {
            this.stream.setPosition((long)this.cursor);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void writeEntry() {
        try {
            this.stream.setPosition(this.stream.getSize());
            this.stream.writeInt(this.cursor);
            this.stream.writeString("BINI");
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public boolean gotoEntry(final boolean isRead) {
        boolean ret = false;
        try {
            if (isRead) {
                this.cursor = 0;
                if (this.stream.getSize() < 8L) {
                    return ret;
                }
                this.stream.setPosition(this.stream.getSize() - 4L);
                final String s = this.stream.readString(4);
                if (s.equals("BINI")) {
                    this.stream.setPosition(this.stream.getSize() - 8L);
                    this.cursor = this.stream.readInt();
                    this.stream.setPosition((long)this.cursor);
                    return true;
                }
            }
            else {
                this.stream.setPosition((long)this.cursor);
                ret = true;
            }
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public void writeInfoBoolean(final boolean value) {
        try {
            this.stream.writeBool(value);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void writeInfoDouble(final double value) {
        try {
            this.stream.writeDouble(value);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void writeInfoInteger(final int value) {
        try {
            this.stream.writeInt(value);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void writeInfoLong(final long value) {
        try {
            this.stream.writeLong(value);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void writeInfoString(final String value) {
        try {
            if (value == null) {
                this.stream.write((byte)0);
                return;
            }
            final byte[] b = this.stream.encodeString(value);
            final byte sl = (byte)b.length;
            this.stream.write(sl);
            this.stream.write(b, 0, b.length);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void writeInfoByte(final byte value) {
        try {
            this.stream.write(value);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public boolean readInfoBoolean() {
        boolean ret = false;
        try {
            ret = this.stream.readBool();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public double readInfoDouble() {
        double ret = 0.0;
        try {
            ret = this.stream.readDouble();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int readInfoInteger() {
        int ret = 0;
        try {
            ret = this.stream.readInt();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public long readInfoLong() {
        long ret = 0L;
        try {
            ret = this.stream.readLong();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public String readInfoString() {
        String ret = null;
        try {
            final byte sl = this.stream.read();
            ret = this.stream.readString((int)sl);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public byte readInfoByte() {
        byte ret = 0;
        try {
            ret = this.stream.read();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int writeBoolean(final boolean value) {
        final int ret = this.cursor;
        try {
            this.stream.setPosition((long)this.cursor);
            this.stream.writeBool(value);
            ++this.cursor;
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int writeDouble(final double value) {
        final int ret = this.cursor;
        try {
            this.stream.setPosition((long)this.cursor);
            this.stream.writeDouble(value);
            this.cursor = (int)this.stream.getPosition();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int writeInteger(final int value) {
        final int ret = this.cursor;
        try {
            this.stream.setPosition((long)this.cursor);
            this.stream.writeInt(value);
            this.cursor = (int)this.stream.getPosition();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int writeLong(final long value) {
        final int ret = this.cursor;
        try {
            this.stream.setPosition((long)this.cursor);
            this.stream.writeLong(value);
            this.cursor = (int)this.stream.getPosition();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int writeString(final String value) {
        final int ret = this.cursor;
        try {
            this.stream.setPosition((long)this.cursor);
            this.stream.writeStringWithSize(value);
            this.cursor = (int)this.stream.getPosition();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public boolean readBoolean(final int position) {
        boolean ret = false;
        try {
            this.stream.setPosition((long)position);
            ret = this.stream.readBool();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public double readDouble(final int position) {
        double ret = 0.0;
        try {
            this.stream.setPosition((long)position);
            ret = this.stream.readDouble();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int readInteger(final int position) {
        int ret = 0;
        try {
            this.stream.setPosition((long)position);
            ret = this.stream.readInt();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public long readLong(final int position) {
        long ret = 0L;
        try {
            this.stream.setPosition((long)position);
            ret = this.stream.readLong();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public String readString(final int position) {
        String ret = null;
        try {
            this.stream.setPosition((long)position);
            final int sl = this.stream.readInt();
            ret = this.stream.readString(sl);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public int writeStream(final Stream aStream, final int size) {
        final int ret = this.cursor;
        try {
            this.stream.setPosition((long)this.cursor);
            this.stream.copyFrom(aStream, (long)size);
            this.cursor = (int)this.stream.getPosition();
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
        return ret;
    }
    
    @Override
    public void readStream(final Stream aStream, final int position, final int size) {
        try {
            this.stream.setPosition((long)position);
            aStream.copyFrom(this.stream, (long)size);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void loadFromStream(final Stream aStream) {
        try {
            this.stream.setSize(aStream.getSize());
            this.stream.setPosition(0L);
            this.stream.copyFrom(aStream, 0L);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    @Override
    public void replaceStream(final Stream aStream) {
        this.stream = aStream;
    }
    
    @Override
    public void saveToStream(final Stream aStream) {
        try {
            aStream.setSize(this.stream.getSize());
            aStream.setPosition(0L);
            aStream.copyFrom(this.stream, 0L);
        }
        catch (final Exception ex) {
            ex.printStackTrace();
        }
    }
    
    public static boolean verify(final Stream stream) {
        try {
            final long origin = stream.getPosition();
            try {
                final StreamIniBuffer buffer = new StreamIniBuffer(stream);
                return buffer.verify();
            }
            finally {
                stream.setPosition(origin);
            }
        }
        catch (final StreamException e) {
            e.printStackTrace();
            return false;
        }
    }
}
